clear; format compact;

% Parameters
t = 0.3;  % assuming a realistic value when in units of eV
% see function 'BA_bands_chem': t_perp = t_perp0 + t_perp1*(cos(kx)-cos(ky)).^2;
t_perp0 = 1.42*t; 
t_perp1 = 0;
filling = 1;

J = 1.93*t;
J_perp = J*(t_perp0/t).^2;  
Matsu_cutoff = 16; 

%% Plot FS
% FS calculation specific parameters
latticesize = 100;
T = 80*8.61732814974056E-05; % temperature 

PlotBA_FS(latticesize,t,t_perp0,t_perp1,T,filling)

%% Calculate DOS
% DOS calculation specific parameters
latticesize = 200;
broadening = 0.03;
num_energy_points = 1000; 
T = 80*8.61732814974056E-05; % temperature 

[DOS0_tot] = CalcAndPlot_DOS(latticesize,t,t_perp0,t_perp1,T,filling,broadening,num_energy_points);

%% Method 1: Tc from linearized gap Eq. 
% Strategy: calculate self energies for Delta = 0 and then apply Eq. (18)

% Parameters for method 1
latticesize = 100;
T_start = 70*8.61732814974056E-05; % initial temperature
imp_points = 50;
imp_strength = linspace(0,0.01,imp_points); % corresponds to n_imp*W^2
symmetry = 'perp'; % choose symmetry from {d,s,perp}
% parameters for iteration of self energy
maxIterations = 5000; 
tolerance = 1e-10; % tolerance 
%-----------------------------

Tc_vec = Calc_Tc_ImpurityDependenc(latticesize,t,t_perp0,t_perp1,J,J_perp,T_start,filling,Matsu_cutoff,imp_strength,symmetry,maxIterations,tolerance);

Plot_Tc_Impurity_Curve_1(imp_strength,Tc_vec,DOS0_tot)

%% Method 2: Tc from full gap eq. 
% Strategy: use full gap eq., starting with highest temperature,
% impurity strength is enhanced until residual gap is below certain threshold. 
% Start at threshold and see whether gap values evolve to higher
% or lower value. If increasing, stop above certain value even without 
% full convergence to avoid unnecessary calculations. Then repeat for next 
% lowest temperature starting with same impurity strength. We assume that
% chemical potential is temperature and impurity dependence is negligible.
%
% ! Typically number of maximum iterations can be small (order 10), because
%   evoultion is typically monotonic. However, some cases require higher 
%   values (which increases computational time significantly)
% ! Starting temperature should be above T_c0.

% Parameters for method 2
latticesize = 100;
T_steps = 100;
T_start = 90; % in Kelvin
T_min = 5;    % in Kelvin
T_vec = linspace(T_min,T_start,T_steps)*8.61732814974056E-05; % temperature T in eV 
energy_cutoff = Inf; % if lowered, higher J, J_perp needed for similar Tc 
% for iteration--------
maxIterations = 25; 
tolerance = 1e-8;
gap_thresh = 1e-5;  % threshold below Tc is set
 % impurity parameters--
imp_max_step_x = 0.1; 
imp_num_step = 6000;
symmetry = 'perp'; % Choose from {'d','perp,'s','mixed'}

V = -3*J;
V_perp = -3/8*J_perp;

[kx,ky,k] = GenerateMesh(latticesize,[0 pi]);
[epsilon_b,epsilon_a] = BA_bands(kx,ky,t,t_perp0,t_perp1,T_vec(end),filling);

% Preallocate
imp_strength_vec = zeros(size(T_vec));
color_vec = zeros(length(T_vec),3);
Delta_vec = zeros(length(T_vec),3);

tic
for i_T = 1:length(T_vec)
    T = T_vec(end - i_T + 1); % start at the end
    disp(['T = ', num2str(T/8.61732814974056E-05)]);

    % Prepare frequency part
    num_points_Matsu = round(Matsu_cutoff/(2*pi*T) - 0.5 + 1);
    omega_n_vec = 2*pi*T*( (0:num_points_Matsu-1) + 1/2);  

    % find points within energy cutoff
    points = (abs(epsilon_b)<energy_cutoff) | (abs(epsilon_a)<energy_cutoff);

    % define quantities frequency-momentum mesh with listed momenta
    list_Eig_b = epsilon_b(points);
    list_Eig_a = epsilon_a(points);
    list_kx = kx(points);
    list_ky = ky(points);
    omega_n = squeeze(OuterProduct(omega_n_vec,ones(size(list_kx))));
    list_Eig_b = squeeze(OuterProduct(ones(size(omega_n_vec)),list_Eig_b));
    list_Eig_a = squeeze(OuterProduct(ones(size(omega_n_vec)),list_Eig_a));
    list_kx = squeeze(OuterProduct(ones(size(omega_n_vec)),list_kx));
    list_ky = squeeze(OuterProduct(ones(size(omega_n_vec)),list_ky));

    % Define gap symmetry functions
    gamma_s = FormFactor(list_kx,list_ky,'s');
    gamma_d = FormFactor(list_kx,list_ky,'dx2y2');
    gamma_perp= FormFactor(list_kx,list_ky,'perp');

    impurity_strength = linspace(imp_strength_vec(max([i_T-1,1])),imp_strength_vec(max([i_T-1,1])) + imp_max_step_x, imp_num_step);
    for i_imp = 1:length(impurity_strength)
        imp_strength_t = impurity_strength(i_imp);        
        disp(['imp_step ', num2str(i_imp), ' with impurity strength ', num2str(imp_strength_t)]);                

        % Starting values for each impurity cycle
        Sigma_0 = 0.0*ones(num_points_Matsu,1);
        Sigma_3 = 0.0*ones(num_points_Matsu,1);
        Sigma_1 = 0.0*ones(num_points_Matsu,1);
        switch symmetry
            case 'd'
                Delta_s = 0.0;       
                Delta_d = gap_thresh;     
                Delta_perp = 0.0;
            case 'perp'
                Delta_s = 0.0;       
                Delta_d = 0.00;     
                Delta_perp = gap_thresh;
            case 's'
                Delta_s = gap_thresh;       
                Delta_d = 0.00;     
                Delta_perp = 0.000;
            case 'mixed'
                Delta_s = gap_thresh;       
                Delta_d = gap_thresh;     
                Delta_perp = gap_thresh;    
        end
        for i = 1:maxIterations
            Delta_b = Delta_s*gamma_s + Delta_d*gamma_d + Delta_perp*gamma_perp; 
            Delta_a = Delta_s*gamma_s + Delta_d*gamma_d - Delta_perp*gamma_perp;            
            
            % Update renormalized quantities
            omega_t = (omega_n + 1i*Sigma_0);    
            Eig_b_t = list_Eig_b + Sigma_3;     
            Eig_a_t = list_Eig_a + Sigma_3;        
            Delta_b_t = Delta_b + Sigma_1;
            Delta_a_t = Delta_a + Sigma_1;

            % Calculate denominator for self-energies
            denom_b = 1./(omega_t.^2 + Eig_b_t.^2 + Delta_b_t.^2);
            denom_a = 1./(omega_t.^2 + Eig_a_t.^2 + Delta_a_t.^2);
            
            % Calculate self energy components Eq.(14)-(16)
            prefac_Sigma = 0.5*imp_strength_t./latticesize.^2;
            Sigma_0 = -1i*prefac_Sigma*sum(omega_t.*denom_b+omega_t.*denom_a,2);
            Sigma_3 = -prefac_Sigma.*sum(Eig_b_t.*denom_b+Eig_a_t.*denom_a,2);
            Sigma_1 = prefac_Sigma.*sum(Delta_b_t.*denom_b+Delta_a_t.*denom_a,2); 

            % Update superconducting order parameter Eq.(11)/(12)
            prefac_gap = T./latticesize.^2;
            switch symmetry
                case 'd'
                    Delta_d    = -V.*prefac_gap.*sum(gamma_d.*       (Delta_b_t.*denom_b + Delta_a_t.*denom_a),'all');         
                case 's'
                    Delta_s    = -V.*prefac_gap*sum(gamma_s.*        (Delta_b_t.*denom_b + Delta_a_t.*denom_a),'all'); 
                case 'perp'
                    Delta_perp = -V_perp.*prefac_gap*sum(gamma_perp.*(Delta_b_t.*denom_b - Delta_a_t.*denom_a),'all');  
                case 'mixed'
                    Delta_d    = -V.*prefac_gap.*sum(gamma_d.*       (Delta_b_t.*denom_b + Delta_a_t.*denom_a),'all');         
                    Delta_s    = -V.*prefac_gap*sum(gamma_s.*        (Delta_b_t.*denom_b + Delta_a_t.*denom_a),'all'); 
                    Delta_perp = -V_perp.*prefac_gap*sum(gamma_perp.*(Delta_b_t.*denom_b - Delta_a_t.*denom_a),'all'); 
            end

            % Convergence check
            Delta_b = Delta_s*gamma_s + Delta_d*gamma_d + Delta_perp*gamma_perp; 
            Delta_a = Delta_s*gamma_s + Delta_d*gamma_d - Delta_perp*gamma_perp;
            precision = mean(abs(omega_t-(omega_n+1i*Sigma_0)),'all') + mean(abs(Eig_b_t-(list_Eig_b+Sigma_3)),'all') + mean(abs(Eig_a_t-(list_Eig_a+Sigma_3)),'all') + mean(abs(Delta_b_t-(Delta_b+Sigma_1)),'all') + mean(abs(Delta_a_t-(Delta_a+Sigma_1)),'all');

            if i == maxIterations
                warning(['No convergence, precision after maximum iterations ', num2str(precision),' is more than tolerance, value is '  num2str([Delta_s, Delta_d, Delta_perp])]);
            elseif i>1    % do at least a single step        
                if precision < tolerance
                   disp(['converged after', num2str(i), 'steps with value', num2str([Delta_s, Delta_d, Delta_perp])]); 
                   break;
                end    
                if (abs(Delta_s) < 0.7*gap_thresh)  && (abs(Delta_perp) < 0.7*gap_thresh) && (abs(Delta_d) < 0.7*gap_thresh)
                    warning(['far below limit after', num2str(i), 'steps with value ', num2str([Delta_s, Delta_d, Delta_perp]),' and precision ', num2str(abs(precision))]);                
                    break;
                elseif abs(Delta_s) > 1.5*gap_thresh || abs(Delta_perp) > 1.5*gap_thresh || abs(Delta_d) > 1.5*gap_thresh
                    disp(['far above limit after', num2str(i), 'steps with value ', num2str([Delta_s, Delta_d, Delta_perp]),' and precision ', num2str(abs(precision))]); 
                    break;                
                end            
            end 
        end % end of iteration for fixed impurity and temperature

        % stopping criteria impurity loop
        if (abs(Delta_s) < gap_thresh)  && (abs(Delta_perp) < gap_thresh) && (abs(Delta_d) < gap_thresh)
            imp_strength_vec(i_T) = imp_strength_t;
            color_vec(i_T,:) = [0 1 0].*abs(Delta_d) + [1 0 1].*abs(Delta_perp) + [1 0 0].*abs(Delta_s); 
            color_vec(i_T,:) = color_vec(i_T,:)./max(color_vec(i_T,:),[],'all');
            Delta_vec(i_T,:) = [Delta_s,Delta_d,Delta_perp];
            disp(['converged with ', num2str(imp_strength_t), ' residual value:' num2str([Delta_s, Delta_d, Delta_perp])]);
            break;
        else
            imp_strength_vec(i_T) = NaN; 
        end
    end % impurity loop
end % temperature loop
toc

%% Plot results
% remove temperatures above T_c0
for i = 1:length(T_vec)
    if imp_strength_vec(i) == 0 && imp_strength_vec(i+1) ~= 0
            T_vec_r = T_vec(1:end-i);
            imp_strength_vec_r = imp_strength_vec(i+1:end);
            color_vec_r = color_vec(i+1:end,:);
            break;
    end
end

Plot_Tc_Impurity_curve_2(imp_strength_vec_r,T_vec_r,color_vec_r,DOS0_tot)


%% Functions---------------------------------------------------------------
% functions of main body
function [] = PlotBA_FS(latticesize,t,t_perp0,t_perp1,T,filling)
    k = linspace(-pi,pi,latticesize);  
    [kx, ky] = meshgrid(k);    
       
    [epsilon_b,epsilon_a] = BA_bands(kx,ky,t,t_perp0,t_perp1,T,filling);   
        
    figure(); hold on;
    contour(kx/pi,ky/pi,epsilon_b,[0,0 ],'color','blue','linewidth',2)
    contour(kx/pi,ky/pi,epsilon_a,[0,0 ],'color','red','linestyle',':','linewidth',2)
    axis square; 
    Lim = [-1,1];
    xlim(Lim); ylim(Lim)
    xticks([-1,0,1]); xticklabels({'-\pi','0','\pi'}); 
    yticks([-1,0,1]); yticklabels({'-\pi','0','\pi'}); 
    set(gca,'box','on');
end
function [DOS0_tot] = CalcAndPlot_DOS(latticesize,t,t_perp0,t_perp1,T,filling,broadening,num_energy_points)
% Only returns density of states at omega = 0 but shows entire plot
  

    [kx,ky] = GenerateMesh(latticesize,[-pi pi]);    
    [epsilon_b,epsilon_a] = BA_bands(kx,ky,t,t_perp0,t_perp1,T,filling);

    energy_min = min([epsilon_b, epsilon_a],[],'all') - 0.2; 
    energy_max = max([epsilon_b, epsilon_a],[],'all') + 0.2; 
    energy_vec = linspace(energy_min,energy_max,num_energy_points); 

    prefactor = 1/(pi*latticesize^2);

    DOS_b = zeros(1,num_energy_points);
    DOS_a = zeros(1,num_energy_points);
    for i_imp = 1:num_energy_points
        DOS_b(i_imp) = prefactor*imag(sum(1./(epsilon_b - energy_vec(i_imp) - 1i*broadening),'all'));
        DOS_a(i_imp) = prefactor*imag(sum(1./(epsilon_a - energy_vec(i_imp) - 1i*broadening),'all'));
    end
    DOS0_b = prefactor*imag(sum(1./(epsilon_b - 1i*broadening),'all'));
    DOS0_a = prefactor*imag(sum(1./(epsilon_a - 1i*broadening),'all'));

    figure(); hold on;
    scatter(0,DOS0_b,'b')
    scatter(0,DOS0_a,'r')
    plot(energy_vec,DOS_b,'b')
    plot(energy_vec,DOS_a,'r')
    xlabel('\omega (eV)')
    ylabel('\rho(\omega) (eV^{-1})')
    DOS0_tot = DOS0_a + DOS0_b;
end
function Tc_vec = Calc_Tc_ImpurityDependenc(latticesize,t,t_perp0,t_perp1,J,J_perp,T_t,filling,Matsu_cutoff,imp_strength,symmetry,maxIterations,tolerance)
Tc_vec = zeros(size(imp_strength));
V = -3*J;   % effective pairing
V_perp = -3/8*J_perp;

[kx,ky,k] = GenerateMesh(latticesize,[0 pi]); 
[epsilon_b,epsilon_a] = BA_bands(kx,ky,t,t_perp0,t_perp1,T_t,filling);

tic
for i_imp = 1:length(imp_strength)
    imp_strength_t = imp_strength(i_imp);
    gap_eq = @(T) Find_Tc(T,k,epsilon_b,epsilon_a,Matsu_cutoff,symmetry,imp_strength_t,maxIterations,tolerance,V,V_perp) ;
    T_c = fzero(gap_eq,T_t);    
    T_t = T_c;

    disp(['impstrength ' num2str(imp_strength_t) ': Tc = ' num2str(T_c/8.61732814974056E-05) ' Kelvin'])
    if T_c < 1*8.61732814974056E-05
        disp(['stopped because ', num2str(T_c/8.61732814974056E-05), ' is below 1 Kelvin']);
        break
    else
       Tc_vec(i_imp) = T_c;
    end
end
toc
end
% for band structure, fix filling
function [epsilon_b,epsilon_a] = BA_bands(kx,ky,t,t_perp0,t_perp1,T,filling) 
% Calculate bonding and antibonding bands for given filling by starting
% with mu = 0 and then adapting 
    mu = 0;
    [epsilon_b,epsilon_a] = BA_bands_chem(kx,ky,t,t_perp0,t_perp1,mu);

    if filling ~= 1 % for half filling, mu = 0 is exact
        epsilon_merged = zeros([2,size(kx)]);
        epsilon_merged(1,:,:) = epsilon_b;
        epsilon_merged(2,:,:) = epsilon_a;
    
        mu = Fix_filling(filling,T,epsilon_merged,0);
    end

    [epsilon_b,epsilon_a] = BA_bands_chem(kx,ky,t,t_perp0,t_perp1,mu);
end
function [epsilon_b,epsilon_a] = BA_bands_chem(kx,ky,t,t_perp0,t_perp1,mu) 
% Calculate bonding and antibonding bands for given chemical potential mu
    t_perp = t_perp0 + t_perp1*(cos(kx)-cos(ky)).^2;    
    epsilon_para = -2*t*(cos(kx) + cos(ky)) - mu;
    epsilon_b = epsilon_para + t_perp;
    epsilon_a = epsilon_para - t_perp;    
end
function filling = Calc_Filling(bands,T)
    if T ~= 0
        filling =  mean(1 - tanh(bands/(2*T)),'all');
    else
        filling = 2* mean((bands<0),'all') + 0.5*mean(bands==0,'all');
    end
end
function mu = Fix_filling(target_filling,T,bands0,mu)
% Adapt chemical potential mu iteratively to set filling to target value
% starting from initial value
    maxIterations = 1000;
    tolerance = 1e-3;


    for i = 1:maxIterations
        bands = bands0 - mu;
        filling_t = Calc_Filling(bands,T);      
        conv_speed = 0.1*rand; % randomness avoids trapping
        mu = mu + conv_speed*(target_filling - filling_t);    
        if abs(target_filling - filling_t) < tolerance
            break;
        end
    end
    if i == maxIterations
        warning(['Could not converge chemical potential. Residual error:', num2str(abs(target_filling - filling_t))]);
    end
end
% for Tc calculation
function output = Find_Tc(T,k,epsilon_b,epsilon_a,Matsu_cutoff,symmetry,imp_strength_t,maxIterations,tolerance,V,V_perp)  
    if T < 1*8.61732814974056E-05
        output = 0;
        return
    end
    latticesize = length(k);

    num_points_Matsu = round(Matsu_cutoff/(2*pi*T) - 0.5 + 1);
    omega_n_vec = 2*pi*T*( (0:num_points_Matsu-1) + 1/2);

    [omega_n,kx,ky] = ndgrid(omega_n_vec, k, k); % define mesh

    epsilon_b = squeeze(OuterProduct(ones(size(omega_n_vec)),epsilon_b));
    epsilon_a = squeeze(OuterProduct(ones(size(omega_n_vec)),epsilon_a));

    % Define gap symmetry functions
    gamma_s = FormFactor(kx,ky,'s');
    gamma_d = FormFactor(kx,ky,'dx2y2');
    gamma_perp = FormFactor(kx,ky,'perp');
    
    Sigma_0 = 0.0*ones(num_points_Matsu,1); 
    Sigma_3 = 0.0*ones(num_points_Matsu,1); 
    for i = 1:maxIterations
        omegaTilde_t = (omega_n + 1i*Sigma_0);    
        epsTilde_b_t = epsilon_b + Sigma_3;     
        epsTilde_a_t = epsilon_a + Sigma_3;
        denom_b = 1./(omegaTilde_t.^2 + epsTilde_b_t.^2);
        denom_a = 1./(omegaTilde_t.^2 + epsTilde_a_t.^2);
        Sigma_0 = -1i*0.5*imp_strength_t.*sum(omegaTilde_t.*denom_b + omegaTilde_t.*denom_a,[2,3])./latticesize.^2;
        Sigma_3 = -0.5*imp_strength_t.*sum(epsTilde_b_t.*denom_b + epsTilde_a_t.*denom_a,[2,3])./latticesize.^2;   
        precision = mean(abs(omegaTilde_t-(omega_n + 1i*Sigma_0)),'all') + mean(abs(epsTilde_b_t-(epsilon_b+Sigma_3)),'all') + mean(abs(epsTilde_a_t-(epsilon_a+Sigma_3)),'all');
        if precision < tolerance
            break;
        end
    end

    % Solve linearized Tc
    prefactor = T./latticesize.^2;
    switch symmetry
        case 'd'
            output = 1 + V.*prefactor.*sum(gamma_d.^2.*(denom_b+denom_a),'all'); % Eq. (11), Sigma1 = 0
        case 'perp'
            % Using Eq. (18) + Eq. (12)
            Omega_b = 0.5*imp_strength_t.*sum(gamma_perp.*(denom_b),[2,3])./latticesize.^2;  
            Omega_a = 0.5*imp_strength_t.*sum(gamma_perp.*(denom_a),[2,3])./latticesize.^2;  
            fac_b = (1-2*Omega_a)./(1-Omega_a-Omega_b);
            fac_a = (1-2*Omega_b)./(1-Omega_a-Omega_b);
            output = 1 + V_perp.*prefactor.*sum(gamma_perp.^2.*(denom_b.*fac_b+denom_a.*fac_a),'all'); 
        case 's'
            % Using Eq. (18) + Eq. (12)
            Omega = 0.5*imp_strength_t.*sum(gamma_s.*(denom_b+denom_a),[2,3])./latticesize.^2;  % non-magnetic point-like impurity            
            output = 1 + V.*prefactor.*sum(gamma_s.^2./(1-Omega).*(denom_b+denom_a),'all');
    end
end
function output = FormFactor(kx,ky,Symmetry)
% form factors for different components
    switch Symmetry
        case 'dx2y2'
            output = 0.5*(cos(kx)-cos(ky));
        case 's'
            output = 0.5*(cos(kx)+cos(ky));
        case 'perp'
            output = 1;
    end
end
% For Plotting results
function [] = Plot_Tc_Impurity_Curve_1(imp_strength,Tc_vec,DOS0_tot)
figure(); hold on;
xdata = imp_strength/max(Tc_vec)*DOS0_tot;
ydata = Tc_vec/max(Tc_vec);
scatter(xdata,ydata)
box on; xlabel('n_{imp}W^2N(0)/T_{c0}'); ylabel('T_c/T_{c0}')
yline(0)
end
function [] = Plot_Tc_Impurity_curve_2(imp_strength_vec,T_vec,color_vec,DOS0_tot)
    xdata = flip(imp_strength_vec)/max(T_vec)*DOS0_tot;
    ydata = T_vec/max(T_vec);
    color_data = flip(color_vec);

    figure(); hold on;
    scatter(xdata,ydata,[],color_data,'Marker','*')
    box on; xlabel('n_{imp}W^2N(0)/{T_c0}'); ylabel('T_c/{T_c0}')
end
% Others
function [x,y,k,k_dist] = GenerateMesh(N,ILim)
%Generate meshgrid to discretize eigenvalues etc.
    k_dist = (ILim(2)-ILim(1))/N;
    k = ILim(1) + ((1:N)-0.5)*k_dist(1);
    [x, y] = ndgrid(k);
end
function C = OuterProduct(A, B) 
C = reshape(A(:) * B(:).', [size(A), size(B)]);
end

