clear; format compact;

% Choose model:
% 'Luo': model from Luo et al., see https://doi.org/10.1103/PhysRevLett.131.126001
% 'Luo-wogamma': model from Luo et al. but with gamma band 50 meV below FS
% 'SingelOrbitalLimit': for checking consistency with SingleOrbital
%                       code, here parameters must be changed inside functions
%                       'BA_bands_chem' and Create_BDG_Hami
Model = 'Luo';   

% general Parameters
Matsu_cutoff = 16; 
switch Model
    case {'Luo','Luo-gamma'}
        filling = 1.5;
        J_perp = 1.93;
        J = J_perp/1.7284;
    case 'SingleOrbitalLimit'
        filling = 1;
        t = .3;  % assuming a realistic value when in units of eV
        t_perp0 = 1.42*t; 
        J = 1.93*t;
        J_perp = J*(t_perp0/t).^2;  
end

%% Plot FS
% FS calculation specific parameters
latticesize = 100;
T = 80*8.61732814974056E-05; % temperature 

PlotBA_FS(latticesize,T,filling,Model)

%% Calculate DOS
% DOS calculation specific parameters
latticesize = 600;
broadening = 0.006;
num_energy_points = 1000; 
T = 80*8.61732814974056E-05; % temperature 

[DOS0_tot] = CalcAndPlot_DOS(latticesize,T,filling,broadening,num_energy_points,Model);

%% Disordered state, Tc suppression
% Strategy: use full gap eq., starting with highest temperature,
% impurity strength is enhanced until residual gap is below certain threshold. 
% Start at threshold and see whether gap values evolve to higher
% or lower value. If increasing, stop above certain value even without 
% full convergence to avoid unnecessary calculations. Then repeat for next 
% lowest temperature starting with same impurity strength. We assume that
% chemical potential is temperature and impurity dependence is negligible.
%
% ! Typically number of maximum iterations can be small (order 10), because
%   evoultion is typically monotonic. However, some cases require higher 
%   values (which increases computational time significantly)
% ! Starting temperature should be above T_c0.


% specific parameters----------------------------
Symmetry = 'perp';
latticesize = 20;
T_steps = 100;
T_start = 80; % in Kelvin
T_stop = 4;   % in Kelvin
T_vec = linspace(T_stop,T_start,T_steps).*8.61732814974056E-05; % temperature in eV  

switch Model
    case {'Luo','Luo-gamma'}
        energy_cutoff = .1;             
    case 'SingleOrbitalLimit'
        energy_cutoff = Inf;
end

% for iteration--------
maxIterations = 10; 
tolerance = 5e-8;  % tolerance 
gap_thresh = 1e-5; % threshold of gap value, if below, we set T_c

% impurity parameters--
imp_max_step_x = 0.1; 
imp_num_step = 12000;

imp_x = 1;
imp_z = 1;
imp_mat = [imp_x 0; 0 imp_z]; 
imp_mat = blkdiag(imp_mat,-imp_mat); % in Nambu space
%------------------------------------------------

% effective interaction
V = -3*J;
V_perp = -3/8*J_perp;

[kx,ky] = GenerateMesh(latticesize,[0 pi]);
[eigs_b,eigs_a, mu] = BA_bands(kx,ky,T_vec(end),filling,Model); 

% create list with k-points with at least one band below energy cutoff
points = (squeeze(abs(eigs_b(1,:,:)))<energy_cutoff) | (squeeze(abs(eigs_b(2,:,:)))<energy_cutoff) ...
       | (squeeze(abs(eigs_a(1,:,:)))<energy_cutoff) | (squeeze(abs(eigs_a(2,:,:)))<energy_cutoff);
list_kx_cut  = kx(points);
list_ky_cut  = ky(points);

% Preallocate
imp_strength_vec = zeros(size(T_vec));
precision_vec = zeros(size(T_vec));
color_vec = zeros(length(T_vec),3);
Delta_vec = zeros(length(T_vec),3); % 'Gap' used for matrices, 'Delta' for scalar components
G_b_vec = zeros(4,4,length(list_kx_cut),T_steps); % interacting bonding Greens function
G_a_vec = zeros(4,4,length(list_kx_cut),T_steps);

tic
for i = 1:length(T_vec)
    T = T_vec(end-i+1);
    disp(['T = ', num2str(T/8.61732814974056E-05)]);
    
    % Prepare frequency part
    num_points_Matsu = round(Matsu_cutoff/(2*pi*T) - 0.5 + 1);
    omega_n_vec = 2*pi*T*( (0:num_points_Matsu-1) + 1/2);  

    % define quantities frequency-momentum mesh with listed momenta
    omega_n = OuterProduct(eye(4),squeeze(OuterProduct(omega_n_vec,ones(size(list_kx_cut)))));
    list_kx_t = squeeze(OuterProduct(ones(size(omega_n_vec)),list_kx_cut));
    list_ky_t = squeeze(OuterProduct(ones(size(omega_n_vec)),list_ky_cut));

    gamma_s = FormFactor(list_kx_t,list_ky_t,'s');
    gamma_d = FormFactor(list_kx_t,list_ky_t,'dx2y2');

    Sigma = zeros(4,4,num_points_Matsu);
    imp_strength_test = linspace(imp_strength_vec(max([i-1,1])),imp_strength_vec(max([i-1,1])) + imp_max_step_x, imp_num_step);
    for j = 1:length(imp_strength_test)
        imp_strength_t = imp_strength_test(j);        
        disp(['imp_step ', num2str(j), ' with impurity strength ', num2str(imp_strength_t)]);                
        
        % starting values
        switch Symmetry
            case 'd' 
                Delta_s = 0;
                Delta_d = gap_thresh;
                Delta_perp = 0;
            case 'perp'             
                Delta_s = 0;
                Delta_d = 0;
                Delta_perp = gap_thresh;
            case 'mixed'
                Delta_s = gap_thresh;
                Delta_d = gap_thresh;
                Delta_perp = gap_thresh;
        end

        
        for i2 = 1:maxIterations     
            Gap_perp = zeros([2,2,size(list_kx_t)]); 
            Gap_para = zeros([2,2,size(list_kx_t)]);    
            switch Symmetry
                case 'd' 
                    Gap_para(1,1,:,:) = Delta_d.*gamma_d;
                case 'perp'             
                    Gap_perp(2,2,:,:) = Delta_perp;
                case 'mixed'
                    switch Model
                        case 'Lou'
                            Gap_para(1,1,:,:) = Delta_d.*gamma_d + Delta_s.*gamma_s;
                            Gap_perp(2,2,:,:) = Delta_perp;
                        case 'SingleOrbitalLimit'
                            Gap_para(1,1,:,:) = Delta_d.*gamma_d + Delta_s.*gamma_s;
                            Gap_para(2,2,:,:) = Delta_d.*gamma_d + Delta_s.*gamma_s;
                            Gap_perp(1,1,:,:) = Delta_perp;
                            Gap_perp(2,2,:,:) = Delta_perp;
                    end
            end

            % Calculate Hamiltonian
            [Hami_b_0,Hami_a_0] = Create_BDG_Hami(list_kx_t,list_ky_t,Gap_para,Gap_perp,Model,mu);
            G0b_inv = 1i*omega_n - Hami_b_0; 
            G0a_inv = 1i*omega_n - Hami_a_0;
            G_b = pageinv(G0b_inv - Sigma); % Eq.(8)
            G_a = pageinv(G0a_inv - Sigma);

            G_b_summed = sum(G_b,4)/latticesize.^2;    
            G_a_summed = sum(G_a,4)/latticesize.^2;

            Sigma_t = 0.5.*imp_strength_t.*(pagemtimes(pagemtimes(imp_mat,G_b_summed+G_a_summed),imp_mat)); % Eq.(7)

            % gap Eq.(21)/(22)
            prefactor = T./latticesize.^2;
            switch Symmetry
                case 'd' 
                    Delta_d_t = prefactor.*V.*sum(real(gamma_d.*(squeeze(G_b(1,3,:,:)) +  squeeze(G_a(1,3,:,:)))) ,'all');
                    Delta_s_t = 0; Delta_perp_t = 0;
                case 'perp'             
                    Delta_perp_t = prefactor.*V_perp.*sum(real((squeeze(G_b(2,4,:,:)) - squeeze(G_a(2,4,:,:)))) ,'all');
                    Delta_s_t = 0; Delta_d_t = 0;
                case 'mixed'
                    Delta_d_t = prefactor.*V.*sum(real(gamma_d.*(squeeze(G_b(1,3,:,:)) +  squeeze(G_a(1,3,:,:)))) ,'all');
                    Delta_s_t = prefactor.*V.*sum(real(gamma_s.*(squeeze(G_b(1,3,:,:)) +  squeeze(G_a(1,3,:,:)))) ,'all');
                    Delta_perp_t = prefactor.*V_perp.*sum(real((squeeze(G_b(2,4,:,:)) -  squeeze(G_a(2,4,:,:)))) ,'all');
            end
            conv_speed = 1; 
            Delta_d = conv_speed.* Delta_d_t + (1-conv_speed).* Delta_d ;
            Delta_s = conv_speed.* Delta_s_t + (1-conv_speed).* Delta_s ;
            Delta_perp = conv_speed.* Delta_perp_t + (1-conv_speed).* Delta_perp ;  
            
            % convergence check
            precision = sum(abs(Sigma-Sigma_t),'all') + abs(Delta_d-Delta_d_t) + abs(Delta_s-Delta_s_t) + abs(Delta_perp-Delta_perp_t) ;  
            if i2 == maxIterations
                warning(['No convergence, precision after max iterations ', num2str(precision),' is more than tolerance, value is ' num2str([Delta_s, Delta_d, Delta_perp])]);
            elseif i2>1    % do atleast a single step        
                if precision < tolerance
                   disp(['converged after ', num2str(i2), ' steps with value ', num2str([Delta_s, Delta_d, Delta_perp])]); 
                   break;
                end    
                if (abs(Delta_s) < 0.7*gap_thresh)  && (abs(Delta_perp) < 0.7*gap_thresh) && (abs(Delta_d) < 0.7*gap_thresh)
                    warning(['far below limit after ', num2str(i), ' steps with value ', num2str([Delta_s, Delta_d, Delta_perp]),' and precision ', num2str(abs(precision))]);                
                    break;
                elseif abs(Delta_s) > 1.5*gap_thresh || abs(Delta_perp) > 1.5*gap_thresh || abs(Delta_d) > 1.5*gap_thresh
                    disp(['far above limit after ', num2str(i), ' steps with value ', num2str([Delta_s, Delta_d, Delta_perp]),' and precision ', num2str(abs(precision))]); 
                    break;                
                end            
            end
            Sigma = Sigma_t;
        end

        % stopping criteria impurity loop
        if (abs(Delta_s) < gap_thresh)  && (abs(Delta_perp) < gap_thresh) && (abs(Delta_d) < gap_thresh)
            imp_strength_vec(i) = imp_strength_t;
            precision_vec(i) = precision;
            color_vec(i,:) = [0 1 0].*abs(Delta_d) + [1 0 1].*abs(Delta_perp) + [1 0 0].*abs(Delta_s); 
            color_vec(i,:) = color_vec(i,:)./max(color_vec(i,:),[],'all');
            Delta_vec(i,:) = [Delta_s,Delta_d,Delta_perp];
            G_b_vec(:,:,:,i) = squeeze(G_b(:,:,1,:)); % save lowest Matsubara component
            G_a_vec(:,:,:,i) = squeeze(G_a(:,:,1,:));
            disp(['stopped with ', num2str(imp_strength_t), ' residual value: ' num2str([Delta_s, Delta_d, Delta_perp])]);          
            break;
        else
            imp_strength_vec(i) = NaN;
        end
    end
end
toc

%% Plot results
% remove temperatures above T_c0 for plotting
for cnt = 1:length(T_vec)
    if imp_strength_vec(cnt) == 0 && imp_strength_vec(cnt+1) ~= 0
        T_vec_r = T_vec(1:end-cnt);
        imp_strength_vec_r = imp_strength_vec(cnt+1:end);
        color_vec_r = color_vec(cnt+1:end,:);
        G_b_vec_r = G_b_vec(:,:,:,cnt+1:end);
        G_a_vec_r = G_a_vec(:,:,:,cnt+1:end);
        break;
    end
end

Plot_Tc_Impurity_curve(imp_strength_vec_r,T_vec_r,color_vec_r,DOS0_tot)

i_comp = 1; % Choose component, i=1 corresponds to lowest impurity strength
Plot_GapStructure(G_b_vec_r,G_a_vec_r,list_kx_cut,list_ky_cut,i_comp)


%% Functions---------------------------------------------------------------
% functions of main body
function [] = PlotBA_FS(latticesize,T,filling,Model)
    k = linspace(-pi,pi,latticesize);  
    [kx, ky] = meshgrid(k);    
       
    [eigs_b,eigs_a] = BA_bands(kx,ky,T,filling,Model);   
    size(eigs_b)    
    figure(); hold on;
    for i = 1:2
    contour(kx/pi,ky/pi,squeeze(eigs_b(i,:,:)),[0,0 ],'color','blue','linewidth',2)
    contour(kx/pi,ky/pi,squeeze(eigs_a(i,:,:)),[0,0 ],'color','red','linestyle',':','linewidth',2)
    end
    axis square; 
    Lim = [-1,1];
    xlim(Lim); ylim(Lim)
    xticks([-1,0,1]); xticklabels({'-\pi','0','\pi'}); 
    yticks([-1,0,1]); yticklabels({'-\pi','0','\pi'}); 
    set(gca,'box','on');
end
function [DOS0_tot] = CalcAndPlot_DOS(latticesize,T,filling,broadening,num_energy_points,Model)
% Only returns density of states at omega = 0 but shows entire plot
  
    [kx,ky] = GenerateMesh(latticesize,[-pi pi]);    
    [eigs_b,eigs_a] = BA_bands(kx,ky,T,filling,Model);

    energy_min = min([eigs_b, eigs_a],[],'all') - 0.2; 
    energy_max = max([eigs_b, eigs_a],[],'all') + 0.2; 
    energy_vec = linspace(energy_min,energy_max,num_energy_points); 

    prefactor = 1/(pi*latticesize^2);

    DOS_b = zeros(2,num_energy_points);
    DOS_a = zeros(2,num_energy_points);
    DOS0_b = [0 0];
    DOS0_a = [0 0];
    for i = 1:2
        for j = 1:num_energy_points
            DOS_b(i,j) = prefactor*imag(sum(1./(squeeze(eigs_b(i,:,:)) - energy_vec(j) - 1i*broadening),'all'));
            DOS_a(i,j) = prefactor*imag(sum(1./(squeeze(eigs_a(i,:,:)) - energy_vec(j) - 1i*broadening),'all'));
        end
    DOS0_b(i) = prefactor*imag(sum(1./(squeeze(eigs_b(i,:,:)) - 1i*broadening),'all'));
    DOS0_a(i) = prefactor*imag(sum(1./(squeeze(eigs_a(i,:,:)) - 1i*broadening),'all'));
    end
    figure(); hold on;
    scatter(0,DOS0_b,'b')
    scatter(0,DOS0_a,'r')
    plot(energy_vec,DOS_b,'b')
    plot(energy_vec,DOS_a,'r')
    xlabel('\omega (eV)')
    ylabel('\rho(\omega) (eV^{-1})')

    DOS0_tot = sum(DOS0_a + DOS0_b);
    switch Model
        case 'SingleOrbitalLimit'
            DOS0_tot = DOS0_tot/2;
    end
end
function [] = Plot_GapStructure(G_b_vec_r,G_a_vec_r,list_kx_cut,list_ky_cut,i_comp)
% plots structure of gaps at Tc


myColormap = [[linspace(0,1)'; ones(100,1)], ...
              [linspace(0,1)'; linspace(1,0)'],...
              [ones(100,1); linspace(1,0)']];

% Read out different orbital components from anomalous part of GF
G_b_zz = G_b_vec_r(2,4,:,i_comp); 
G_a_zz = G_a_vec_r(2,4,:,i_comp);
F_zz = real(squeeze(G_b_zz+G_a_zz));
G_b_xx = G_b_vec_r(1,3,:,i_comp);
G_a_xx = G_a_vec_r(1,3,:,i_comp);
F_xx = real(squeeze(G_b_xx+G_a_xx));
G_b_xz = G_b_vec_r(1,4,:,i_comp);
G_a_xz = G_a_vec_r(1,4,:,i_comp);
F_xz = real(squeeze(G_b_xz+G_a_xz));

colorbar_max = max(abs([F_zz,F_xx,F_xz]),[],'all'); % for normalization
F_zz = F_zz/colorbar_max;
F_xx = F_xx/colorbar_max;
F_xz = F_xz/colorbar_max;

% Set up figure
figure('Position',[100 100 650 260]); 
Hsize = 0.9;
Wsize = 0.26;
botPos = 0.03;
leftPos  = 0.08;
midPos   = leftPos + Wsize + 0.06;
rightPos = midPos + Wsize + 0.06;

Pos1 = [leftPos botPos Wsize Hsize];
axes('Position',Pos1,'Layer','top'); hold on

scatter(list_kx_cut,list_ky_cut,10,F_zz,'filled');
scatter(-list_kx_cut,list_ky_cut,10,F_zz,'filled');
scatter(list_kx_cut,-list_ky_cut,10,F_zz,'filled');
scatter(-list_kx_cut,-list_ky_cut,10,F_zz,'filled');
xlim([-pi pi])
ylim([-pi pi])
xticks(pi*[-1 -0.5 0 0.5 1])
yticks(pi*[-1 -0.5 0 0.5 1])
xticklabels({'-\pi' '-\pi/2' 0 '\pi/2' '\pi'})
yticklabels({'-\pi' '-\pi/2' 0 '\pi/2' '\pi'})
axis square; box on; colormap(myColormap); clim([-1 1])
xlabel('k_x')
ylabel('k_y')
title('F_{zz}')

Pos2 = [midPos botPos Wsize Hsize];
axes('Position',Pos2,'Layer','top'); hold on

scatter(list_kx_cut,list_ky_cut,10,F_xx,'filled');
scatter(-list_kx_cut,list_ky_cut,10,F_xx,'filled');
scatter(list_kx_cut,-list_ky_cut,10,F_xx,'filled');
scatter(-list_kx_cut,-list_ky_cut,10,F_xx,'filled');
xlim([-pi pi])
ylim([-pi pi])
xticks(pi*[-1 -0.5 0 0.5 1])
yticks(pi*[-1 -0.5 0 0.5 1])
xticklabels({'-\pi' '-\pi/2' 0 '\pi/2' '\pi'})
yticklabels({'-\pi' '-\pi/2' 0 '\pi/2' '\pi'})
axis square; box on; colormap(myColormap); clim([-1 1])
xlabel('k_x')
title('F_{xx}')

Pos3 = [rightPos botPos Wsize Hsize];
axes('Position',Pos3,'Layer','top'); hold on

scatter(list_kx_cut,list_ky_cut,10,F_xz,'filled');
scatter(-list_kx_cut,list_ky_cut,10,F_xz,'filled');
scatter(list_kx_cut,-list_ky_cut,10,F_xz,'filled');
scatter(-list_kx_cut,-list_ky_cut,10,F_xz,'filled');
xlim([-pi pi])
ylim([-pi pi])
xticks(pi*[-1 -0.5 0 0.5 1])
yticks(pi*[-1 -0.5 0 0.5 1])
xticklabels({'-\pi' '-\pi/2' 0 '\pi/2' '\pi'})
yticklabels({'-\pi' '-\pi/2' 0 '\pi/2' '\pi'})
axis square; box on; colormap(myColormap); clim([-1 1])
xlabel('k_x')
title('F_{xz}')

end
function [] = Plot_Tc_Impurity_curve(imp_strength_vec,T_vec,color_vec,DOS0_tot)
    xdata = flip(imp_strength_vec)/max(T_vec)*DOS0_tot;
    ydata = T_vec/max(T_vec);
    color_data = flip(color_vec);

    figure(); hold on;
    scatter(xdata,ydata,[],color_data,'Marker','*')
    box on; xlabel('n_{imp}W^2N(0)/{T_c0}'); ylabel('T_c/{T_c0}')
end
% for band structure, fix filling
function [eigs_b,eigs_a, mu] = BA_bands(kx,ky,T,filling,Model) 
% Calculate bonding and antibonding bands for given filling by starting
% with mu = 0 and then adapting 
    mu = 0;
    [eigs_b,eigs_a] = BA_bands_chem(kx,ky,mu,Model);

    if filling ~= 1.5 % , mu = 0 is exact
        eigs_merged = zeros([4,size(kx)]);
        eigs_merged(1:2,:,:) = eigs_b;
        eigs_merged(3:4,:,:) = eigs_a;
    
        mu = Fix_filling(filling,T,eigs_merged,0);
    end

    [eigs_b,eigs_a] = BA_bands_chem(kx,ky,mu,Model);
end
function [eigs_b,eigs_a] = BA_bands_chem(kx,ky,mu,Model) 
% Calculate bonding and antibonding bands for given chemical potential mu
    switch Model
        case 'Luo'
            ez = 0.409;      
        case 'Luo-wogamma'
            mu = mu - 0.307;   % shifted such that mu = 0 corresponds to filling of 1.5
            ez = -0.095 - mu;
    end
    switch Model
        case {'Luo','Luo-wogamma'}
            ex = 0.779 - mu;
            tx1 = -0.483; 
            tz1 = -0.11;
            tx2 =  0.069;
            tz2 = -0.017;
            txz3 = 0.239;
            tx_perp = 0.005;
            tz_perp = -0.635;
            txz4 = -0.034;
        case 'SingleOrbitalLimit'
            t = 0.3;
            t_perp0 = 1.42*t; 
            t_perp1 = 0;

            ez = -mu; 
            ex = -mu; 
            tx1 = t; 
            tz1 = t;
            tx2 = 0;
            tz2 = 0;
            txz3 = 0;

            tx_perp = t_perp0 + t_perp1*(cos(kx)-cos(ky)).^2;
            tz_perp = t_perp0 + t_perp1*(cos(kx)-cos(ky)).^2;
            txz4 = 0.0;
    end
    Tx = 2*tx1*(cos(kx) + cos(ky)) + 4*tx2*cos(kx).*cos(ky) + ex;
    Tz = 2*tz1*(cos(kx) + cos(ky)) + 4*tz2*cos(kx).*cos(ky) + ez;
    V = 2*txz3*(cos(kx) - cos(ky));
    Vprime = 2*txz4*(cos(kx) - cos(ky));

    eigs_b = zeros([2,size(kx')]);
    eigs_a = zeros([2,size(kx')]);

    D_b = sqrt(((Tx-Tz + (tx_perp-tz_perp))/2).^2 + (V+Vprime).^2 );
    D_a = sqrt(((Tx-Tz - (tx_perp-tz_perp))/2).^2 + (V-Vprime).^2 );

    eigs_b(1,:,:) = (Tx+Tz + (tx_perp+tz_perp))/2 + D_b;
    eigs_b(2,:,:) = (Tx+Tz + (tx_perp+tz_perp))/2 - D_b;

    eigs_a(1,:,:) = (Tx+Tz - (tx_perp+tz_perp))/2 + D_a;
    eigs_a(2,:,:) = (Tx+Tz - (tx_perp+tz_perp))/2 - D_a;

    % expressions for transformation
    % phase_b = sign(V+Vprime+10^(-18)); phase_a = sign(V-Vprime+10^(-18));    % small positive shifts avoid that signum gives 0 for vanishing argument
    % Ub11 = sqrt(0.5)*sqrt((1+(Tx-Tz+(tx_perp-tz_perp))./(2.*D_b))).*phase_b;
    % Ub12 = sqrt(0.5)*sqrt((1-(Tx-Tz+(tx_perp-tz_perp))./(2.*D_b)));
    % Ua11 = sqrt(0.5)*sqrt((1+(Tx-Tz-(tx_perp-tz_perp))./(2.*D_a))).*phase_a;
    % Ua12 = sqrt(0.5)*sqrt((1-(Tx-Tz-(tx_perp-tz_perp))./(2.*D_a)));
    % 
    % U_b = zeros([2,2,size(kx')]);
    % U_a = zeros([2,2,size(kx')]);
    % 
    % U_b(1,1,:,:) =  Ub11;
    % U_b(1,2,:,:) =  Ub12;
    % U_b(2,1,:,:) =  Ub12;
    % U_b(2,2,:,:) = -Ub11;
    % 
    % U_a(1,1,:,:) =  Ua11;
    % U_a(1,2,:,:) =  Ua12;
    % U_a(2,1,:,:) =  Ua12;
    % U_a(2,2,:,:) = -Ua11;
end
function filling = Calc_Filling(bands,T)
    if T ~= 0
        filling =  mean(1 - tanh(bands/(2*T)),'all');
    else
        filling = 2* mean((bands<0),'all') + 0.5*mean(bands==0,'all');
    end
end
function [Hami_b,Hami_a] = Create_BDG_Hami(kx,ky,Delta,Delta_perp,Model,mu)
    switch Model
        case 'Luo'
            ez = 0.409;      
        case 'Luo-wogamma'
            mu = mu - 0.307;   % shifted such that mu = 0 corresponds to filling of 1.5
            ez = -0.095 - mu;
    end
    switch Model
        case {'Luo','Luo-wogamma'}
            ex = 0.779 - mu;
            tx1 = -0.483; 
            tz1 = -0.11;
            tx2 =  0.069;
            tz2 = -0.017;
            txz3 = 0.239;
            tx_perp = 0.005;
            tz_perp = -0.635;
            txz4 = -0.034;
        case 'SingleOrbitalLimit'
            t = 0.3;
            t_perp0 = 1.42*t; 
            t_perp1 = 0;

            ez = -mu; 
            ex = -mu; 
            tx1 = t; 
            tz1 = t;
            tx2 = 0;
            tz2 = 0;
            txz3 = 0;

            tx_perp = t_perp0 + t_perp1*(cos(kx)-cos(ky)).^2;
            tz_perp = t_perp0 + t_perp1*(cos(kx)-cos(ky)).^2;
            txz4 = 0.0;
    end

    Tx = 2*tx1*(cos(kx) + cos(ky)) + 4*tx2*cos(kx).*cos(ky) + ex;
    Tz = 2*tz1*(cos(kx) + cos(ky)) + 4*tz2*cos(kx).*cos(ky) + ez;
    V = 2*txz3*(cos(kx) - cos(ky));
    Vprime = 2*txz4*(cos(kx) - cos(ky));

    Hami_b = zeros([4,4,size(kx)]);
    Hami_a = zeros([4,4,size(kx)]);
    
    Hami_b(1,1,:,:) = Tx + tx_perp;
    Hami_b(2,2,:,:) = Tz + tz_perp;
    Hami_b(1,2,:,:) = V + Vprime; 
    Hami_b(2,1,:,:) = V + Vprime; 
    Hami_b(3:4,3:4,:,:) = -Hami_b(1:2,1:2,:,:);
    Hami_b(1:2,3:4,:,:) = Delta + Delta_perp;
    Hami_b(3:4,1:2,:,:) = pagetranspose(Delta+Delta_perp);

    Hami_a(1,1,:,:) = Tx - tx_perp;
    Hami_a(2,2,:,:) = Tz - tz_perp;
    Hami_a(1,2,:,:) = V - Vprime; 
    Hami_a(2,1,:,:) = V - Vprime; 
    Hami_a(3:4,3:4,:,:) = -Hami_a(1:2,1:2,:,:);
    Hami_a(1:2,3:4,:,:) = Delta-Delta_perp;
    Hami_a(3:4,1:2,:,:) = pagetranspose(Delta-Delta_perp);
end
function mu = Fix_filling(target_filling,T,bands0,mu)
% Adapt chemical potential mu iteratively to set filling to target value
% starting from initial value
    maxIterations = 1000;
    tolerance = 1e-3;


    for i = 1:maxIterations
        bands = bands0 - mu;
        filling_t = Calc_Filling(bands,T);      
        conv_speed = 0.1*rand; % randomness avoids trapping
        mu = mu + conv_speed*(target_filling - filling_t);    
        if abs(target_filling - filling_t) < tolerance
            break;
        end
    end
    if i == maxIterations
        warning(['Could not converge chemical potential. Residual error:', num2str(abs(n - filling_n))]);
    end
end
function output = FormFactor(kx,ky,Symmetry)
% form factors for different components
    switch Symmetry
        case 'dx2y2'
            output = 0.5*(cos(kx)-cos(ky));
        case 's'
            output = 0.5*(cos(kx)+cos(ky));
        case 'perp'
            output = 1;
    end
end
% Others
function [x,y,k,k_dist] = GenerateMesh(N,ILim)
%Generate meshgrid to discretize eigenvalues etc.
    k_dist = (ILim(2)-ILim(1))/N;
    k = ILim(1) + ((1:N)-0.5)*k_dist(1);
    [x, y] = ndgrid(k);
end
function C = OuterProduct(A, B) 
C = reshape(A(:) * B(:).', [size(A), size(B)]);
end

